/** @file
This library contains functions to provide an onTick() and onBar() eventhandler.<br>
The EA or the indicator has to implement two procedures:<br> 
events.onBar(int pPeriodIndex) and events.onTick()<br><br>
To actually use the Eventhandler then the EA has to implement into init()
\code
void init() {
	...
	// Initialize Eventhandler to provide onBar and onTick Events.
	events.initEvents();
	...
}
\endcode
<br>
and in the start() procedure of the EA:<br>
\code
int start() {
	...
	// Perform eventhandler calls.
	events.doEvents();
	...
	return(0);
}
\endcode

Templates for those procedures are here:<br><br>
\code
void events.onBar(int pPeriod) {
	switch (pPeriod) {
	case PERIOD_M1:
		break;
	case PERIOD_M5:
		break;
	case PERIOD_M15:
		break;
	case PERIOD_M30:
		break;
	case PERIOD_H1:
		break;
	case PERIOD_H4:
		break;
	case PERIOD_D1:
		break;
	case PERIOD_W1:
		break;
	default:
		break;
	}
	return;
}
\endcode
<br>
and for the onTick() procedure:
<br>
\code
void events.onTick() {
	// check whether a trade had been closed since.
	hist.checkClose(openTrades);
	// Calculate Stops and close orders
	modifyStops();
	// Calculate signals according to the indicators
	calculateSignal();
	// Buy or sell according to the signals
	doTrade();
}
\endcode

*/
#define MIN_TIMEFRAME 0
#define MAX_TIMEFRAMES 9
static datetime events.timeFrames[MAX_TIMEFRAMES];

static int events.periodIndex2Period[]= {
    PERIOD_M1,  
	PERIOD_M5,  
	PERIOD_M15, 
	PERIOD_M30, 
	PERIOD_H1,  
	PERIOD_H4,
    PERIOD_D1,  
	PERIOD_W1,  
	PERIOD_MN1
};

void events.initEvents() {
	Print("initEvents: Intializing Eventhandler $Rev: 483 $");
	//
	// initially populate timeFrames
	for(int i=MIN_TIMEFRAME; i<MAX_TIMEFRAMES; i++) {
		events.timeFrames[i] = iTime(NULL, events.periodIndex2Period[i], 0);
	}
}

void events.doEvents() {
	// Check whether a new bar in an other Timeframe has to be called...
	for(int i=MIN_TIMEFRAME; i<MAX_TIMEFRAMES; i++) {
		if (events.timeFrames[i] != iTime(NULL, events.periodIndex2Period[i], 0) ) {
			/* Print("doEvents: Detected "
				+ " period " + events.periodIndex2Period[i]
				+ " before/after " + events.timeFrames[i] + "/" + iTime(NULL, events.periodIndex2Period[i], 0)
			); */
			events.timeFrames[i]= iTime(NULL, events.periodIndex2Period[i], 0);
			events.onBar(events.periodIndex2Period[i]);
		}
	}
	// Now process each tick.
	events.onTick();
}

//+------------------------------------------------------------------+
//| Helper function to calculate an array index of the given timeframe
//+------------------------------------------------------------------+
int events.periodToPeriodIndex(int periodIndex) {
	switch(periodIndex) {
	case PERIOD_M1  :
		return(0);
		break;
	case PERIOD_M5  :
		return(1);
		break;
	case PERIOD_M15 :
		return(2);
		break;
	case PERIOD_M30 :
		return(3);
		break;
	case PERIOD_H1  :
		return(4);
		break;
	case PERIOD_H4  :
		return(5);
		break;
	case PERIOD_D1  :
		return(6);
		break;
	}
}

//+------------------------------------------------------------------+
//| Helper function to calculate a timeframe of the given array index
//+------------------------------------------------------------------+
int events.periodIndexToPeriod(int pIndex) {
	return(events.periodIndex2Period[pIndex]);
}
